import { ThreeObject3d } from '@angular-three/core';
import { Directive, EventEmitter, Input, Optional, Output, SkipSelf, } from '@angular/core';
import { AudioListenerDirective } from '../audio-listener';
export class ThreeAudio {
    constructor(listenerDirective, parentObjectDirective) {
        this.listenerDirective = listenerDirective;
        this.parentObjectDirective = parentObjectDirective;
        this.ready = new EventEmitter();
    }
    ngOnInit() {
        if (!this.listenerDirective) {
            console.error('AudioListener must exist to initialize Audio');
            return;
        }
        this._audio = new this.audioType(this.listenerDirective.audioListener);
        if (this.audio) {
            if (this.object3d) {
                this.object3d.add(this.audio);
            }
            else if (this.parentObjectDirective) {
                this.parentObjectDirective.object3d.add(this.audio);
            }
        }
        this.ready.emit(this.audio);
    }
    get audio() {
        return this._audio;
    }
}
ThreeAudio.decorators = [
    { type: Directive }
];
ThreeAudio.ctorParameters = () => [
    { type: AudioListenerDirective, decorators: [{ type: Optional }, { type: SkipSelf }] },
    { type: ThreeObject3d, decorators: [{ type: Optional }, { type: SkipSelf }] }
];
ThreeAudio.propDecorators = {
    object3d: [{ type: Input }],
    ready: [{ type: Output }]
};
//# sourceMappingURL=audio.abstract.js.map