import { CanvasStore, DestroyedService } from '@angular-three/core';
import { Directive, EventEmitter, Input, NgZone, Output, SkipSelf, } from '@angular/core';
import { takeUntil } from 'rxjs/operators';
import { AudioListener } from 'three';
export class AudioListenerDirective {
    constructor(canvasStore, destroyed, ngZone) {
        this.canvasStore = canvasStore;
        this.destroyed = destroyed;
        this.ngZone = ngZone;
        this.ready = new EventEmitter();
    }
    ngOnInit() {
        this.ngZone.runOutsideAngular(() => {
            this._listener = new AudioListener();
            if (this.filter) {
                this._listener.filter = this.filter;
            }
            if (this.timeDelta) {
                this._listener.timeDelta = this.timeDelta;
            }
            this.canvasStore.active$
                .pipe(takeUntil(this.destroyed))
                .subscribe((active) => {
                this.ngZone.runOutsideAngular(() => {
                    const { camera } = this.canvasStore.getImperativeState();
                    if (active && camera) {
                        camera.add(this.audioListener);
                        this.ready.emit(this.audioListener);
                    }
                });
            });
        });
    }
    get audioListener() {
        return this._listener;
    }
}
AudioListenerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-audio-listener',
                exportAs: 'ngtAudioListener',
                providers: [DestroyedService],
            },] }
];
AudioListenerDirective.ctorParameters = () => [
    { type: CanvasStore, decorators: [{ type: SkipSelf }] },
    { type: DestroyedService },
    { type: NgZone }
];
AudioListenerDirective.propDecorators = {
    filter: [{ type: Input }],
    timeDelta: [{ type: Input }],
    ready: [{ type: Output }]
};
//# sourceMappingURL=audio-listener.directive.js.map