import { Directive, EventEmitter, Inject, Input, NgZone, Optional, Output, SkipSelf, } from '@angular/core';
import { Object3D } from 'three';
import { AnimationLoopParticipant, ThreeObject3d } from '../abstracts';
import { Object3dControllerDirective } from '../controllers';
import { OBJECT_3D_CONTROLLER_PROVIDER, OBJECT_3D_WATCHED_CONTROLLER, } from '../di';
import { AnimationStore, CanvasStore } from '../stores';
export class PrimitiveDirective extends AnimationLoopParticipant {
    constructor(animationStore, ngZone, object3dController, canvasStore, parentObjectDirective) {
        super(animationStore, ngZone);
        this.animationStore = animationStore;
        this.ngZone = ngZone;
        this.object3dController = object3dController;
        this.canvasStore = canvasStore;
        this.parentObjectDirective = parentObjectDirective;
        this.ready = new EventEmitter();
    }
    get object() {
        return this._object;
    }
    set object(value) {
        if (value == null) {
            console.error('[object] is required');
        }
        this._object = value;
        if (value) {
            this.ready.emit(value);
        }
    }
    get object3d() {
        return this._object;
    }
    ngOnInit() {
        this.ngZone.runOutsideAngular(() => {
            this.appendToParent();
        });
    }
    ngOnDestroy() {
        super.ngOnDestroy();
        this.ngZone.runOutsideAngular(() => {
            if (this.object3d) {
                this.remove();
            }
        });
    }
    appendToParent() {
        if (this.object3d && this.object3d instanceof Object3D) {
            if (this.object3dController.appendTo) {
                this.object3dController.appendTo.add(this.object3d);
                return;
            }
            const { scene } = this.canvasStore.getImperativeState();
            if (this.object3dController.appendMode === 'root') {
                if (scene) {
                    scene.add(this.object3d);
                }
                return;
            }
            if (this.object3dController.appendMode === 'immediate') {
                if (this.parentObjectDirective) {
                    this.parentObjectDirective.object3d.add(this.object3d);
                }
                else {
                    if (scene) {
                        scene.add(this.object3d);
                    }
                }
            }
        }
    }
    remove() {
        if (this.object3d instanceof Object3D) {
            if (this.object3dController.appendTo) {
                this.object3dController.appendTo.remove(this.object3d);
            }
            else if (this.parentObjectDirective &&
                this.object3dController.appendMode === 'immediate') {
                this.parentObjectDirective.object3d.remove(this.object3d);
            }
            else {
                const { scene } = this.canvasStore.getImperativeState();
                if (scene) {
                    scene.remove(this.object3d);
                }
            }
            this.object3d.clear();
        }
    }
}
PrimitiveDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-primitive',
                exportAs: 'ngtPrimitive',
                providers: [
                    {
                        provide: ThreeObject3d,
                        useExisting: PrimitiveDirective,
                    },
                    OBJECT_3D_CONTROLLER_PROVIDER,
                ],
            },] }
];
PrimitiveDirective.ctorParameters = () => [
    { type: AnimationStore },
    { type: NgZone },
    { type: Object3dControllerDirective, decorators: [{ type: Inject, args: [OBJECT_3D_WATCHED_CONTROLLER,] }] },
    { type: CanvasStore },
    { type: ThreeObject3d, decorators: [{ type: Optional }, { type: SkipSelf }] }
];
PrimitiveDirective.propDecorators = {
    ready: [{ type: Output }],
    object: [{ type: Input }]
};
//# sourceMappingURL=primitive.directive.js.map