import { Injectable } from '@angular/core';
import { tap } from 'rxjs/operators';
import { createEvents } from '../utils';
import { CanvasStore } from './canvas.store';
import { ImperativeComponentStore } from './imperative-component-store.abstract';
import { InstancesStore } from './instances.store';
const supportedEvents = [
    'click',
    'contextmenu',
    'dblclick',
    'wheel',
    'pointerdown',
    'pointerup',
    'pointerleave',
    'pointermove',
    'pointercancel',
    'lostpointercapture',
];
export class EventsStore extends ImperativeComponentStore {
    constructor(canvasStore, instancesStore) {
        super({
            connected: false,
            internal: {
                interaction: [],
                hovered: new Map(),
                captured: undefined,
                initialClick: [0, 0],
                initialHits: [],
            },
        });
        this.canvasStore = canvasStore;
        this.instancesStore = instancesStore;
        this.initHandlersEffect = this.effect((trigger$) => trigger$.pipe(tap(() => {
            const { handlePointer } = createEvents(() => this.canvasStore.getImperativeState(), () => this.getImperativeState(), () => this.instancesStore.getImperativeState());
            this.patchState({
                handlers: supportedEvents.reduce((handlers, supportedEventName) => (Object.assign(Object.assign({}, handlers), { [supportedEventName]: handlePointer(supportedEventName) })), {}),
            });
        })));
        this.connectEffect = this.effect((target$) => target$.pipe(tap((target) => {
            this.disconnectEffect();
            const { handlers } = this.getImperativeState();
            Object.entries(handlers !== null && handlers !== void 0 ? handlers : []).forEach(([name, event]) => target.addEventListener(name, event, { passive: true }));
            this.patchState({ connected: target });
        })));
        this.disconnectEffect = this.effect((trigger$) => trigger$.pipe(tap(() => {
            const { handlers, connected } = this.getImperativeState();
            if (connected) {
                Object.entries(handlers !== null && handlers !== void 0 ? handlers : []).forEach(([name, event]) => {
                    if (connected instanceof HTMLElement) {
                        connected.removeEventListener(name, event);
                    }
                });
                this.patchState({ connected: false });
            }
        })));
        this.initHandlersEffect();
    }
    ngOnDestroy() {
        this.disconnectEffect();
        super.ngOnDestroy();
    }
}
EventsStore.decorators = [
    { type: Injectable }
];
EventsStore.ctorParameters = () => [
    { type: CanvasStore },
    { type: InstancesStore }
];
//# sourceMappingURL=events.store.js.map