function applyDottedPathProps(instance, key, propAtKey, isRoot = true) {
    const [first, ...paths] = key.split('.');
    const rootChild = instance[first];
    if (rootChild == null) {
        return;
    }
    if (paths.length >= 1) {
        applyDottedPathProps(rootChild, paths.join('.'), propAtKey, false);
    }
    else {
        instance[first] = propAtKey;
    }
    if (isRoot) {
        checkNeedsUpdate(rootChild);
    }
}
function checkNeedsUpdate(value) {
    if (typeof value === 'object' && 'needsUpdate' in value) {
        value['needsUpdate'] = true;
    }
}
/**
 * Apply props on instances
 *
 * @internal
 * @private
 */
export function applyProps(instance, props) {
    if (!props || (props && !Object.keys(props).length))
        return;
    if ('__ngt' in props) {
        instance['__ngt'] = props['__ngt'];
    }
    if (instance['set'] != null &&
        typeof instance['set'] === 'function') {
        instance['set'](props);
    }
    for (const [key, propAtKey] of Object.entries(props)) {
        if (key.split('.').length > 1) {
            applyDottedPathProps(instance, key, propAtKey);
        }
        else {
            if (instance[key] == null) {
                instance[key] = propAtKey;
            }
            else {
                if (instance[key]['set'] !=
                    null &&
                    typeof instance[key]['set'] === 'function') {
                    if (Array.isArray(propAtKey)) {
                        instance[key]['set'](...propAtKey);
                    }
                    else {
                        instance[key]['set'](propAtKey);
                    }
                }
                else {
                    instance[key] = propAtKey;
                }
            }
            checkNeedsUpdate(propAtKey);
        }
    }
}
//# sourceMappingURL=apply-props.util.js.map