import { ThreeObject3d } from '@angular-three/core';
import { ContentChildren, Directive, EventEmitter, Input, NgZone, Optional, Output, SkipSelf, } from '@angular/core';
import { Matrix4, Skeleton } from 'three';
import { BoneDirective } from './bone.directive';
import { SkinnedMeshDirective } from './skinned-mesh.directive';
export class SkeletonDirective {
    constructor(ngZone, hostObject) {
        this.ngZone = ngZone;
        this.hostObject = hostObject;
        this.ready = new EventEmitter();
    }
    ngOnInit() {
        this.ngZone.runOutsideAngular(() => {
            if (this.bones) {
                const boneInverses = this.boneInverses
                    ? this.boneInverses.map((threeMaxtrix) => {
                        if (threeMaxtrix instanceof Matrix4)
                            return threeMaxtrix;
                        return new Matrix4().set(...threeMaxtrix);
                    })
                    : undefined;
                this._skeleton = new Skeleton(this.bones.map((b) => b.object3d), boneInverses);
                this.ready.emit(this.skeleton);
                if (this.hostObject) {
                    if (this.hostObject instanceof SkinnedMeshDirective) {
                        const bindMatrix = this.hostObject.bindMatrix
                            ? this.hostObject.bindMatrix instanceof Matrix4
                                ? this.hostObject.bindMatrix
                                : new Matrix4().set(...this.hostObject.bindMatrix)
                            : undefined;
                        this.hostObject.object3d.bind(this._skeleton, bindMatrix);
                    }
                }
            }
        });
    }
    get skeleton() {
        return this._skeleton;
    }
}
SkeletonDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-skeleton',
                exportAs: 'ngtSkeleton',
            },] }
];
SkeletonDirective.ctorParameters = () => [
    { type: NgZone },
    { type: ThreeObject3d, decorators: [{ type: Optional }, { type: SkipSelf }] }
];
SkeletonDirective.propDecorators = {
    boneInverses: [{ type: Input }],
    ready: [{ type: Output }],
    bones: [{ type: ContentChildren, args: [BoneDirective,] }]
};
//# sourceMappingURL=skeleton.directive.js.map