import { ThreeBufferGeometry, ThreeAttribute } from '@angular-three/core';
import { Directive, NgZone, Optional, Input, NgModule } from '@angular/core';
import { InterleavedBufferAttribute, BufferAttribute, InstancedBufferAttribute, Float16BufferAttribute, Float32BufferAttribute, Float64BufferAttribute, Int8BufferAttribute, Int16BufferAttribute, Int32BufferAttribute, Uint8BufferAttribute, Uint16BufferAttribute, Uint32BufferAttribute, Uint8ClampedBufferAttribute } from 'three';

class InterleavedBufferAttributeDirective {
    constructor(ngZone, geometryDirective) {
        this.ngZone = ngZone;
        this.geometryDirective = geometryDirective;
    }
    ngOnChanges() {
        this.ngZone.runOutsideAngular(() => {
            if (this.attribute) {
                this.attribute.needsUpdate = true;
            }
        });
    }
    ngOnInit() {
        this.ngZone.runOutsideAngular(() => {
            if (this.geometryDirective && this.attach) {
                this._attribute = new InterleavedBufferAttribute(...this.args);
                if (this.attribute) {
                    this.geometryDirective.bufferGeometry.setAttribute(this.attach, this.attribute);
                }
            }
        });
    }
    ngOnDestroy() {
        this.ngZone.runOutsideAngular(() => {
            if (this.geometryDirective && this.attach) {
                this.geometryDirective.bufferGeometry.deleteAttribute(this.attach);
            }
        });
    }
    get attribute() {
        return this._attribute;
    }
}
InterleavedBufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-interleaved-buffer-attribute',
                exportAs: 'ngtInterleavedBufferAttribute',
            },] }
];
InterleavedBufferAttributeDirective.ctorParameters = () => [
    { type: NgZone },
    { type: ThreeBufferGeometry, decorators: [{ type: Optional }] }
];
InterleavedBufferAttributeDirective.propDecorators = {
    args: [{ type: Input }],
    attach: [{ type: Input }]
};

class InterleavedBufferAttributeModule {
}
InterleavedBufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [InterleavedBufferAttributeDirective],
                exports: [InterleavedBufferAttributeDirective],
            },] }
];

// GENERATED
class BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-buffer-attribute',
                exportAs: 'ngtBufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: BufferAttributeDirective,
                    },
                ],
            },] }
];
BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeBufferAttributeModule {
}
ThreeBufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [BufferAttributeDirective],
                exports: [BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class InstancedBufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = InstancedBufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
InstancedBufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-instanced-buffer-attribute',
                exportAs: 'ngtInstancedBufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: InstancedBufferAttributeDirective,
                    },
                ],
            },] }
];
InstancedBufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeInstancedBufferAttributeModule {
}
ThreeInstancedBufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [InstancedBufferAttributeDirective],
                exports: [InstancedBufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Float16BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Float16BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Float16BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-float16-buffer-attribute',
                exportAs: 'ngtFloat16BufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Float16BufferAttributeDirective,
                    },
                ],
            },] }
];
Float16BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeFloat16BufferAttributeModule {
}
ThreeFloat16BufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Float16BufferAttributeDirective],
                exports: [Float16BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Float32BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Float32BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Float32BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-float32-buffer-attribute',
                exportAs: 'ngtFloat32BufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Float32BufferAttributeDirective,
                    },
                ],
            },] }
];
Float32BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeFloat32BufferAttributeModule {
}
ThreeFloat32BufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Float32BufferAttributeDirective],
                exports: [Float32BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Float64BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Float64BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Float64BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-float64-buffer-attribute',
                exportAs: 'ngtFloat64BufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Float64BufferAttributeDirective,
                    },
                ],
            },] }
];
Float64BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeFloat64BufferAttributeModule {
}
ThreeFloat64BufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Float64BufferAttributeDirective],
                exports: [Float64BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Int8BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Int8BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Int8BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-int8-buffer-attribute',
                exportAs: 'ngtInt8BufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Int8BufferAttributeDirective,
                    },
                ],
            },] }
];
Int8BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeInt8BufferAttributeModule {
}
ThreeInt8BufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Int8BufferAttributeDirective],
                exports: [Int8BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Int16BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Int16BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Int16BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-int16-buffer-attribute',
                exportAs: 'ngtInt16BufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Int16BufferAttributeDirective,
                    },
                ],
            },] }
];
Int16BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeInt16BufferAttributeModule {
}
ThreeInt16BufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Int16BufferAttributeDirective],
                exports: [Int16BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Int32BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Int32BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Int32BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-int32-buffer-attribute',
                exportAs: 'ngtInt32BufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Int32BufferAttributeDirective,
                    },
                ],
            },] }
];
Int32BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeInt32BufferAttributeModule {
}
ThreeInt32BufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Int32BufferAttributeDirective],
                exports: [Int32BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Uint8BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Uint8BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Uint8BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-uint8-buffer-attribute',
                exportAs: 'ngtUint8BufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Uint8BufferAttributeDirective,
                    },
                ],
            },] }
];
Uint8BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeUint8BufferAttributeModule {
}
ThreeUint8BufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Uint8BufferAttributeDirective],
                exports: [Uint8BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Uint16BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Uint16BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Uint16BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-uint16-buffer-attribute',
                exportAs: 'ngtUint16BufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Uint16BufferAttributeDirective,
                    },
                ],
            },] }
];
Uint16BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeUint16BufferAttributeModule {
}
ThreeUint16BufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Uint16BufferAttributeDirective],
                exports: [Uint16BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Uint32BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Uint32BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Uint32BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-uint32-buffer-attribute',
                exportAs: 'ngtUint32BufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Uint32BufferAttributeDirective,
                    },
                ],
            },] }
];
Uint32BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeUint32BufferAttributeModule {
}
ThreeUint32BufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Uint32BufferAttributeDirective],
                exports: [Uint32BufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED
class Uint8ClampedBufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = Uint8ClampedBufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
Uint8ClampedBufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-uint8-clamped-buffer-attribute',
                exportAs: 'ngtUint8ClampedBufferAttribute',
                providers: [
                    {
                        provide: ThreeAttribute,
                        useExisting: Uint8ClampedBufferAttributeDirective,
                    },
                ],
            },] }
];
Uint8ClampedBufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

// GENERATED
class ThreeUint8ClampedBufferAttributeModule {
}
ThreeUint8ClampedBufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [Uint8ClampedBufferAttributeDirective],
                exports: [Uint8ClampedBufferAttributeDirective],
            },] }
];

// GENERATED

// GENERATED

/**
 * Generated bundle index. Do not edit.
 */

export { BufferAttributeDirective, Float16BufferAttributeDirective, Float32BufferAttributeDirective, Float64BufferAttributeDirective, InstancedBufferAttributeDirective, Int16BufferAttributeDirective, Int32BufferAttributeDirective, Int8BufferAttributeDirective, InterleavedBufferAttributeDirective, InterleavedBufferAttributeModule, ThreeBufferAttributeModule, ThreeFloat16BufferAttributeModule, ThreeFloat32BufferAttributeModule, ThreeFloat64BufferAttributeModule, ThreeInstancedBufferAttributeModule, ThreeInt16BufferAttributeModule, ThreeInt32BufferAttributeModule, ThreeInt8BufferAttributeModule, ThreeUint16BufferAttributeModule, ThreeUint32BufferAttributeModule, ThreeUint8BufferAttributeModule, ThreeUint8ClampedBufferAttributeModule, Uint16BufferAttributeDirective, Uint32BufferAttributeDirective, Uint8BufferAttributeDirective, Uint8ClampedBufferAttributeDirective };
//# sourceMappingURL=angular-three-core-attributes.js.map
