import { DestroyedService, CanvasStore, ThreeObject3d } from '@angular-three/core';
import { EventEmitter, Directive, SkipSelf, NgZone, Input, Output, NgModule, Optional } from '@angular/core';
import { takeUntil } from 'rxjs/operators';
import { AudioListener, PositionalAudio } from 'three';

class AudioListenerDirective {
    constructor(canvasStore, destroyed, ngZone) {
        this.canvasStore = canvasStore;
        this.destroyed = destroyed;
        this.ngZone = ngZone;
        this.ready = new EventEmitter();
    }
    ngOnInit() {
        this.ngZone.runOutsideAngular(() => {
            this._listener = new AudioListener();
            if (this.filter) {
                this._listener.filter = this.filter;
            }
            if (this.timeDelta) {
                this._listener.timeDelta = this.timeDelta;
            }
            this.canvasStore.active$
                .pipe(takeUntil(this.destroyed))
                .subscribe((active) => {
                this.ngZone.runOutsideAngular(() => {
                    const { camera } = this.canvasStore.getImperativeState();
                    if (active && camera) {
                        camera.add(this.audioListener);
                        this.ready.emit(this.audioListener);
                    }
                });
            });
        });
    }
    get audioListener() {
        return this._listener;
    }
}
AudioListenerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-audio-listener',
                exportAs: 'ngtAudioListener',
                providers: [DestroyedService],
            },] }
];
AudioListenerDirective.ctorParameters = () => [
    { type: CanvasStore, decorators: [{ type: SkipSelf }] },
    { type: DestroyedService },
    { type: NgZone }
];
AudioListenerDirective.propDecorators = {
    filter: [{ type: Input }],
    timeDelta: [{ type: Input }],
    ready: [{ type: Output }]
};

class ThreeAudioListenerModule {
}
ThreeAudioListenerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AudioListenerDirective],
                exports: [AudioListenerDirective],
            },] }
];

class ThreeAudio {
    constructor(listenerDirective, parentObjectDirective) {
        this.listenerDirective = listenerDirective;
        this.parentObjectDirective = parentObjectDirective;
        this.ready = new EventEmitter();
    }
    ngOnInit() {
        if (!this.listenerDirective) {
            console.error('AudioListener must exist to initialize Audio');
            return;
        }
        this._audio = new this.audioType(this.listenerDirective.audioListener);
        if (this.audio) {
            if (this.object3d) {
                this.object3d.add(this.audio);
            }
            else if (this.parentObjectDirective) {
                this.parentObjectDirective.object3d.add(this.audio);
            }
        }
        this.ready.emit(this.audio);
    }
    get audio() {
        return this._audio;
    }
}
ThreeAudio.decorators = [
    { type: Directive }
];
ThreeAudio.ctorParameters = () => [
    { type: AudioListenerDirective, decorators: [{ type: Optional }, { type: SkipSelf }] },
    { type: ThreeObject3d, decorators: [{ type: Optional }, { type: SkipSelf }] }
];
ThreeAudio.propDecorators = {
    object3d: [{ type: Input }],
    ready: [{ type: Output }]
};

class PositionalAudioDirective extends ThreeAudio {
    constructor() {
        super(...arguments);
        this.audioType = PositionalAudio;
    }
}
PositionalAudioDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-positional-audio',
                exportAs: 'ngtPositionalAudio',
            },] }
];

class ThreePositionalAudioModule {
}
ThreePositionalAudioModule.decorators = [
    { type: NgModule, args: [{
                declarations: [PositionalAudioDirective],
                exports: [PositionalAudioDirective],
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { AudioListenerDirective, PositionalAudioDirective, ThreeAudio, ThreeAudioListenerModule, ThreePositionalAudioModule };
//# sourceMappingURL=angular-three-core-audios.js.map
