import { ThreeMesh, ThreeObject3d, OBJECT_3D_CONTROLLER_PROVIDER } from '@angular-three/core';
import { Directive, NgModule, Input, EventEmitter, NgZone, Optional, SkipSelf, Output, ContentChildren } from '@angular/core';
import { Mesh, InstancedMesh, DynamicDrawUsage, SkinnedMesh, Bone, Matrix4, Skeleton } from 'three';

class MeshDirective extends ThreeMesh {
    constructor() {
        super(...arguments);
        this.meshType = Mesh;
    }
}
MeshDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-mesh',
                exportAs: 'ngtMesh',
                providers: [
                    { provide: ThreeObject3d, useExisting: MeshDirective },
                    OBJECT_3D_CONTROLLER_PROVIDER,
                ],
            },] }
];

class ThreeMeshModule {
}
ThreeMeshModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MeshDirective],
                exports: [MeshDirective],
            },] }
];

class InstancedMeshDirective extends ThreeMesh {
    constructor() {
        super(...arguments);
        this.meshType = InstancedMesh;
        this.customize = () => {
            this.object3d.instanceMatrix.setUsage(DynamicDrawUsage);
        };
    }
    set args(v) {
        this.extraArgs = v;
    }
    ngOnChanges() {
        this.init();
        super.ngOnChanges();
    }
    canCreate() {
        return !this.object3d && this.geometry;
    }
}
InstancedMeshDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-instanced-mesh',
                exportAs: 'ngtInstancedMesh',
                providers: [
                    {
                        provide: ThreeObject3d,
                        useExisting: InstancedMeshDirective,
                    },
                    OBJECT_3D_CONTROLLER_PROVIDER,
                ],
            },] }
];
InstancedMeshDirective.propDecorators = {
    args: [{ type: Input }]
};

class ThreeInstancedMeshModule {
}
ThreeInstancedMeshModule.decorators = [
    { type: NgModule, args: [{
                declarations: [InstancedMeshDirective],
                exports: [InstancedMeshDirective],
            },] }
];

class SkinnedMeshDirective extends ThreeMesh {
    constructor() {
        super(...arguments);
        this.meshType = SkinnedMesh;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
SkinnedMeshDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-skinned-mesh',
                exportAs: 'ngtSkinnedMesh',
                providers: [
                    { provide: ThreeObject3d, useExisting: SkinnedMeshDirective },
                    OBJECT_3D_CONTROLLER_PROVIDER,
                ],
            },] }
];
SkinnedMeshDirective.propDecorators = {
    args: [{ type: Input }],
    bindMatrix: [{ type: Input }],
    bindMode: [{ type: Input }]
};

class BoneDirective extends ThreeObject3d {
    ngOnInit() {
        this.init();
    }
    initObject() {
        this._bone = new Bone();
    }
    get object3d() {
        return this._bone;
    }
}
BoneDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-bone',
                exportAs: 'ngtBone',
                providers: [
                    { provide: ThreeObject3d, useExisting: BoneDirective },
                    OBJECT_3D_CONTROLLER_PROVIDER,
                ],
            },] }
];

class SkeletonDirective {
    constructor(ngZone, hostObject) {
        this.ngZone = ngZone;
        this.hostObject = hostObject;
        this.ready = new EventEmitter();
    }
    ngOnInit() {
        this.ngZone.runOutsideAngular(() => {
            if (this.bones) {
                const boneInverses = this.boneInverses
                    ? this.boneInverses.map((threeMaxtrix) => {
                        if (threeMaxtrix instanceof Matrix4)
                            return threeMaxtrix;
                        return new Matrix4().set(...threeMaxtrix);
                    })
                    : undefined;
                this._skeleton = new Skeleton(this.bones.map((b) => b.object3d), boneInverses);
                this.ready.emit(this.skeleton);
                if (this.hostObject) {
                    if (this.hostObject instanceof SkinnedMeshDirective) {
                        const bindMatrix = this.hostObject.bindMatrix
                            ? this.hostObject.bindMatrix instanceof Matrix4
                                ? this.hostObject.bindMatrix
                                : new Matrix4().set(...this.hostObject.bindMatrix)
                            : undefined;
                        this.hostObject.object3d.bind(this._skeleton, bindMatrix);
                    }
                }
            }
        });
    }
    get skeleton() {
        return this._skeleton;
    }
}
SkeletonDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-skeleton',
                exportAs: 'ngtSkeleton',
            },] }
];
SkeletonDirective.ctorParameters = () => [
    { type: NgZone },
    { type: ThreeObject3d, decorators: [{ type: Optional }, { type: SkipSelf }] }
];
SkeletonDirective.propDecorators = {
    boneInverses: [{ type: Input }],
    ready: [{ type: Output }],
    bones: [{ type: ContentChildren, args: [BoneDirective,] }]
};

class ThreeSkinnedMeshModule {
}
ThreeSkinnedMeshModule.decorators = [
    { type: NgModule, args: [{
                declarations: [SkinnedMeshDirective, BoneDirective, SkeletonDirective],
                exports: [SkinnedMeshDirective, BoneDirective, SkeletonDirective],
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { BoneDirective, InstancedMeshDirective, MeshDirective, SkeletonDirective, SkinnedMeshDirective, ThreeInstancedMeshModule, ThreeMeshModule, ThreeSkinnedMeshModule };
//# sourceMappingURL=angular-three-core-meshes.js.map
